@@ -3,6 +3,7 @@
 
 Used to communicate with providers without using CFME facilities
 """
+from datetime import datetime
 import winrm
 import json
 import urlparse
@@ -11,6 +12,7 @@ from cStringIO import StringIO
 from contextlib import contextmanager
 from textwrap import dedent
 
+from exceptions import VMInstanceNotFound
 from lxml import etree
 from wait_for import wait_for
 
@@ -38,6 +40,7 @@ class AzureSystem(MgmtSystemAPIBase):
         self.host = kwargs["powershell_host"]
         self.provisioning = kwargs['provisioning']
         self.resource_group = kwargs['provisioning']['resource_group']
+        self.storage_container = kwargs['provisioning']['storage_container']
         self.username = kwargs["username"]
         self.password = kwargs["password"]
         self.ui_username = kwargs["ui_username"]
@@ -48,6 +51,7 @@ class AzureSystem(MgmtSystemAPIBase):
         self.storage_key = kwargs["storage_key"]
         self.subscription_id = kwargs["subscription_id"]
         self.tenant_id = kwargs["tenant_id"]
+        self.region = kwargs["provisioning"]["region_api"]
         self.api = winrm.Session(self.host, auth=(self.ps_username, self.ps_password))
 
     @property
@@ -65,16 +69,24 @@ class AzureSystem(MgmtSystemAPIBase):
         Get-AzureRmSubscription -SubscriptionId \"{}\" -TenantId \"{}\" | Select-AzureRmSubscription
         """.format(self.ui_username, self.ui_password, self.subscription_id, self.tenant_id))
 
-    def run_script(self, script):
+    def run_script(self, script, ignore_error=False, tries=3):
         """Wrapper for running powershell scripts. Ensures the ``pre_script`` is loaded."""
         script = dedent(script)
-        self.logger.info(" Running PowerShell script:\n{}\nn{}".format(self.pre_script, script))
-        result = self.api.run_ps("{}\n\n{}".format(self.pre_script, script))
-        if result.status_code != 0:
+        while tries > 0:
+            if '\n' not in script:
+                # Save space when no newline in the script
+                self.logger.info(" Running PowerShell script: {}".format(script))
+            else:
+                self.logger.info(" Running PowerShell script:\n{}".format(script))
+            result = self.api.run_ps("{}\n\n{}".format(self.pre_script, script))
+            if result.status_code == 0:
+                self.logger.info("run_script Script Complete")
+                return result.std_out.strip()
+            self.logger.error("Script returned {}!: {}".format(result.status_code, result.std_err))
+            tries -= 1
+        if result.status_code != 0 and not ignore_error:
             raise self.PowerShellScriptError("Script returned {}!: {}"
-                .format(result.status_code, result.std_err))
-        self.logger.info("PowerShell Returned:\n{}\n".format(result.std_out.strip()))
-        return result.std_out.strip()
+                                         .format(result.status_code, result.std_err))
 
     def start_vm(self, vm_name, resource_group=None):
         if self.is_vm_stopped(vm_name, resource_group or self.resource_group):
@@ -138,15 +150,19 @@ class AzureSystem(MgmtSystemAPIBase):
         raise NotImplementedError('NIE - create_vm not implemented.')
 
     def delete_vm(self, vm_name, resource_group=None):
-        current_vhd = self.get_vm_vhd(vm_name, resource_group or self.resource_group)
-        self.logger.info("Attempting to Retrieve the VMs Disk {}".format(current_vhd))
+        self.logger.info("Begin delete_vm {}".format(vm_name))
+        vhd_endpoint = self.get_vm_vhd(vm_name, resource_group or self.resource_group)
         self.run_script(
             """
             Invoke-Command -scriptblock {{
             Remove-AzureRmVM -ResourceGroupName \"{rg}\" -Name \"{vm}\" -Force
-            Remove-AzureRmNetworkInterface -Name \"{vm}\"-ResourceGroupName \"{rg}\" -Force
+            Remove-AzureRmNetworkInterface -Name \"{vm}\" -ResourceGroupName \"{rg}\" -Force
+            Remove-AzureRmPublicIpAddress -Name \"{vm}\" -ResourceGroupName \"{rg}\" -Force
             }}
-            """.format(rg=resource_group or self.resource_group, vm=vm_name))
+            """.format(rg=resource_group or self.resource_group, vm=vm_name), True)
+
+        vhd_name = os.path.split(urlparse.urlparse(vhd_endpoint).path)[1]
+        self.remove_blob_image(vhd_name)
 
     def list_vm(self):
         self.logger.info("Attempting to List Azure VMs")
@@ -156,6 +172,40 @@ class AzureSystem(MgmtSystemAPIBase):
             "./Object/Property[@Name='Name']/text()")
         return nameList
 
+    def capture_vm(self, vm_name, resource_group, container, image_name):
+        self.logger.info("Attempting to Capture Azure VM {}".format(vm_name))
+        self.stop_vm(vm_name, resource_group)
+
+        self.logger.info("Generalizing passed VM {}".format(vm_name))
+        self.run_script('Set-AzureRmVM -ResourceGroupName "{}" '
+                        '-Name "{}" -Generalized'.format(resource_group, vm_name))
+        self.logger.info("The VM {} is generalized".format(vm_name))
+
+        self.run_script('Save-AzureRmVMImage -ResourceGroupName "{}" '
+                        '-VMName "{}" -DestinationContainerName "{}" '
+                        '-VHDNamePrefix "{}"'.format(resource_group, vm_name, container,
+                                                     image_name))
+        # save..image always puts image to system
+        # https://github.com/Azure/azure-powershell/issues/2714
+        if len([image for image in self.list_blob_images('system') if image_name in image]) > 0:
+            self.logger.info("Azure VM {} is captured".format(vm_name))
+        else:
+            raise RuntimeError("image {} isn't found in container {}".format(image_name, container))
+
+    def list_stack(self, resource_group=None):
+        self.logger.info("Attempting to List Azure Orchestration Deployment Stacks")
+        azure_data = self.run_script(
+            """
+            Invoke-Command -scriptblock {{
+            Get-AzureRmResourceGroupDeployment -ResourceGroupName \"{rg}\" |
+            convertto-xml -as String;
+            }}
+            """.format(rg=resource_group or self.resource_group))
+        data = self.clean_azure_xml(azure_data)
+        name_list = etree.parse(StringIO(data)).getroot().xpath(
+            "./Object/Property[@Name='DeploymentName']/text()")
+        return name_list
+
     def list_template(self):
         self.logger.info("Attempting to List Azure VHDs in templates directory")
         azure_data = self.run_script(
@@ -181,8 +231,42 @@ class AzureSystem(MgmtSystemAPIBase):
     def list_network(self):
         raise NotImplementedError('list_network not implemented.')
 
-    def vm_creation_time(self, vm_name):
-        raise NotImplementedError('vm_creation_time not implemented.')
+    def vm_creation_time(self, vm_name, resource_group=None):
+        # There is no such parameter as vm creation time.  Using VHD date instead.
+        self.logger.info("Attempting to Retrieve Azure VM Modification Time {}".format(vm_name))
+        vm_vhd = self.get_vm_vhd(vm_name, resource_group or self.resource_group)
+        vhd_name = os.path.split(urlparse.urlparse(vm_vhd).path)[1]
+        data = self.run_script(
+            """
+            Invoke-Command -scriptblock {{
+            $storageContext = New-AzureStorageContext -StorageAccountName \"{storage_account}\" `
+                            -StorageAccountKey \"{storage_key}\"
+            Get-AzureStorageBlob -Name \"{storage_container}\" `
+                            -Context $storageContext -Blob \"{vhd_blob}\" | convertto-xml -as String
+            }}
+            """.format(storage_account=self.storage_account,
+                       storage_container=self.storage_container,
+                       storage_key=self.storage_key,
+                       vhd_blob=vhd_name), True)
+        vhd_last_modified = etree.parse(StringIO(self.clean_azure_xml(data))).getroot().xpath(
+            "./Object/Property[@Name='LastModified']/text()")
+        create_time = datetime.strptime(str(vhd_last_modified), '[\'%m/%d/%Y %H:%M:%S %p +00:00\']')
+        self.logger.info("VM last edit time based on vhd =  {}".format(str(create_time)))
+        return create_time
+
+    def create_netsec_group(self, group_name, resource_group):
+        self.logger.info("Attempting to Create New Azure Security Group {}".format(group_name))
+        self.run_script(
+            'New-AzureRmNetworkSecurityGroup -Location "{}" -Name "{}" '
+            '-ResourceGroupName "{}"'.format(self.region, group_name, resource_group))
+        self.logger.info("Network Security Group {} is created".format(group_name))
+
+    def remove_netsec_group(self, group_name, resource_group):
+        self.logger.info("Attempting to Remove Azure Security Group {}".format(group_name))
+        self.run_script(
+            'Remove-AzureRmNetworkSecurityGroup -Name "{}" '
+            '-ResourceGroupName "{}" -Force'.format(group_name, resource_group))
+        self.logger.info("Network Security Group {} is removed".format(group_name))
 
     def info(self, vm_name):
         pass
@@ -204,6 +288,10 @@ class AzureSystem(MgmtSystemAPIBase):
         data = self.clean_azure_xml(azure_data)
         statusValue = json.loads(etree.parse(StringIO(data)).getroot().xpath(
             "./Object/Property[@Name='StatusesText']/text()")[0])
+        # If script runs completely but the result isn't the one we need - better to show Azure
+        # message
+        if statusValue[0]['DisplayStatus'] == 'Provisioning failed':
+            raise VMInstanceNotFound(statusValue[0]['Message'])
         powerStatus = statusValue[1]
         powerDisplayStatus = powerStatus['DisplayStatus']
         self.logger.info("Returned Status was {}".format(powerDisplayStatus))
@@ -245,15 +333,25 @@ class AzureSystem(MgmtSystemAPIBase):
         raise NotImplementedError('NIE - clone_vm not implemented.')
 
     def does_vm_exist(self, vm_name):
-        result = self.list_vm()
-        if vm_name in result:
-            return True
-        else:
-            return False
+        return vm_name in self.list_vm()
+
+    def stack_exist(self, stack_name):
+        return stack_name in self.list_stack()
+
+    def delete_stack(self, stack_name, resource_group=None):
+        self.logger.info("Removes a Deployment Stack resource created with Orchestration")
+        self.run_script(
+            """
+            Invoke-Command -scriptblock {{
+            Remove-AzureRmResourceGroupDeployment -ResourceGroupName \"{rg}\" `
+            -DeploymentName \"{stack}\" -Force
+            }}
+            """.format(rg=resource_group or self.resource_group, stack=stack_name))
+        return True
 
     def deploy_template(self, template, vm_name=None, **vm_settings):
         self.copy_blob_image(template, vm_name, vm_settings['storage_account'],
-            vm_settings['template_blob'], vm_settings['storage_blob'])
+            vm_settings['template_container'], vm_settings['storage_container'])
         self.run_script(
             """
             Invoke-Command -scriptblock {{
@@ -264,35 +362,42 @@ class AzureSystem(MgmtSystemAPIBase):
             $PIp = New-AzureRmPublicIpAddress -Name \"{vm_name}\" -ResourceGroupName `
                 \"{resource_group}\" -Location \"{region}\" -AllocationMethod Dynamic -Force
             $SubnetConfig = New-AzureRmVirtualNetworkSubnetConfig -Name "default" `
-                -AddressPrefix "10.0.0.0/24"
+                -AddressPrefix \"{subnet_range}\"
             $VNet = New-AzureRmVirtualNetwork -Name \"{virtual_net}\" -ResourceGroupName `
-                \"{resource_group}\" -Location \"{region}\" -AddressPrefix "10.0.0.0/16" `
+                \"{resource_group}\" -Location \"{region}\" -AddressPrefix \"{address_space}\" `
                 -Subnet $SubnetConfig -Force
             $Interface = New-AzureRmNetworkInterface -Name \"{vm_name}\" -ResourceGroupName `
                 \"{resource_group}\" -Location \"{region}\" -SubnetId $VNet.Subnets[0].Id `
                 -PublicIpAddressId $PIp.Id -Force
             $VirtualMachine = New-AzureRmVMConfig -VMName \"{vm_name}\" -VMSize \"{vm_size}\"
             $VirtualMachine = Add-AzureRmVMNetworkInterface -VM $VirtualMachine -Id $Interface.Id
-            $OSDiskUri = $StorageAccount.PrimaryEndpoints.Blob.ToString() + \"{storage_blob}\" `
+            $OSDiskUri = $StorageAccount.PrimaryEndpoints.Blob.ToString() + \"{storage_cont}\" `
                 + "/" + \"{vm_name}.vhd\"
             $VirtualMachine = Set-AzureRmVMOSDisk -VM $VirtualMachine -Name \"{vm_name}\" `
                 -VhdUri $OSDiskUri -CreateOption attach -Linux
-            New-AzureRmVM -ResourceGroupName \"{resource_group}\" -Location \"{region}\"`
+            New-AzureRmVM -ResourceGroupName \"{resource_group}\" -Location \"{region}\" `
                 -VM $VirtualMachine
             }}
             """.format(source_name=template.split("/")[-1],
                        vm_name=vm_name,
                        resource_group=vm_settings['resource_group'],
                        virtual_net=vm_settings['virtual_net'],
+                       address_space=vm_settings['address_space'],
+                       subnet_range=vm_settings['subnet_range'],
                        network_nsg=vm_settings['network_nsg'],
                        region=vm_settings['region_api'],
                        vm_size=vm_settings['vm_size'],
                        av_set=vm_settings['av_set'],
                        storage_account=vm_settings['storage_account'],
-                       storage_blob=vm_settings['storage_blob']))
+                       storage_cont=vm_settings['storage_container']))
         self.wait_vm_running(vm_name, vm_settings['resource_group'])
 
-    def copy_blob_image(self, template, vm_name, storage_account, template_blob, storage_blob):
+    def copy_blob_image(self,
+                        template,
+                        vm_name,
+                        storage_account,
+                        template_container,
+                        storage_container):
         self.run_script(
             """
             Invoke-Command -scriptblock {{
@@ -300,31 +405,62 @@ class AzureSystem(MgmtSystemAPIBase):
                             -StorageAccountKey \"{storage_key}\"
             $destContext = New-AzureStorageContext -StorageAccountName \"{storage_account}\" `
                             -StorageAccountKey \"{storage_key}\"
-            $blobCopy = Start-AzureStorageBlobCopy -DestContainer \"{storage_blob}\" `
+            $blobCopy = Start-AzureStorageBlobCopy -DestContainer \"{storage_container}\" `
                         -DestContext $destContext -DestBlob \"{vm_name}.vhd\" `
                         -SrcBlob \"{source_name}.vhd\" `
-                        -Context $sourceContext -SrcContainer \"{template_blob}\"
+                        -Context $sourceContext -SrcContainer \"{template_container}\"
             }}
             """.format(source_name=template.split("/")[-1],
                        vm_name=vm_name,
                        storage_account=storage_account,
                        storage_key=self.storage_key,
-                       template_blob=template_blob,
-                       storage_blob=storage_blob))
+                       template_container=template_container,
+                       storage_container=storage_container))
+
+    def remove_blob_image(self, vm_vhd, container=None):
+        if not container:
+            container = self.storage_container
 
-    def remove_blob_image(self, vm_vhd, storage_account, storage_blob):
         self.run_script(
             """
             Invoke-Command -scriptblock {{
             $sourceContext = New-AzureStorageContext -StorageAccountName \"{storage_account}\" `
                             -StorageAccountKey \"{storage_key}\"
-            $blobRemove = Remove-AzureStorageBlob -Blob \"{vm_vhd}\" `
-                        -Context $sourceContext -Container \"{storage_blob}\"
+            $blobRemove = Remove-AzureStorageBlob -Blob \"{vhd_name}\" `
+                        -Context $sourceContext -Container \"{container}\"
             }}
-            """.format(vm_vhd=vm_vhd,
-                       storage_account=storage_account,
+            """.format(vhd_name=vm_vhd,
+                       storage_account=self.storage_account,
                        storage_key=self.storage_key,
-                       storage_blob=storage_blob))
+                       container=container), True)
+
+    def list_blob_images(self, container):
+        azure_data = self.run_script(
+            'New-AzureStorageContext -StorageAccountName "{acc_name}"'
+            ' -StorageAccountKey "{acc_key}"|Get-AzureStorageBlob -Container "{container}"|'
+            'Select Name|convertto-xml -as String'.format(acc_name=self.storage_account,
+                                                          acc_key=self.storage_key,
+                                                          container=container))
+        data = self.clean_azure_xml(azure_data)
+        names = etree.parse(StringIO(data)).getroot().xpath(
+            "./Object/Property[@Name='Name']/text()")
+        return names
+
+    def remove_diags_container(self):
+        self.run_script(
+            """
+            Invoke-Command -scriptblock {{
+            $storeContext = New-AzureStorageContext -StorageAccountName \"{storage_account}\" `
+                            -StorageAccountKey \"{storage_key}\"
+            $storageContainer = Get-AzureStorageContainer -Context $storeContext
+            foreach ($container in $storageContainer) {{
+                if ($container.name -like 'bootdiagnostics-test*'){{
+                    Remove-AzureStorageContainer -Name $container.name -Context $storeContext -Force
+                }}
+            }}
+            }}
+            """.format(storage_account=self.storage_account,
+                       storage_key=self.storage_key), True)
 
     @contextmanager
     def with_vm(self, *args, **kwargs):
@@ -334,29 +470,29 @@ class AzureSystem(MgmtSystemAPIBase):
         self.delete_vm(name)
 
     def current_ip_address(self, vm_name, resource_group=None):
+        # Returns first active IPv4 IpAddress only
         azure_data = self.run_script(
             "Get-AzureRmPublicIpAddress -ResourceGroup \"{}\" -Name \"{}\" |"
             "convertto-xml -as String".format(resource_group or self.resource_group, vm_name))
         data = self.clean_azure_xml(azure_data)
         return etree.parse(StringIO(data)).getroot().xpath(
             "./Object/Property[@Name='IpAddress']/text()")
-        # TODO: Scavenge informations how these are formatted, I see no if-s in SCVMM
 
     def get_ip_address(self, vm_name, resource_group=None, **kwargs):
         current_ip_address = self.current_ip_address(vm_name, resource_group or self.resource_group)
         return current_ip_address
 
     def get_vm_vhd(self, vm_name, resource_group=None):
-        self.logger.info("Attempting to Retrieve Azure VM VHD {}".format(vm_name))
+        self.logger.info("get_vm_vhd - Attempting to Retrieve Azure VM VHD {}".format(vm_name))
         azure_data = self.run_script(
             "Get-AzureRmVm -ResourceGroup \"{}\" -Name \"{}\" | convertto-xml -as String"
             .format(resource_group or self.resource_group, vm_name))
         data = self.clean_azure_xml(azure_data)
-        status_value = json.loads(etree.parse(StringIO(data)).getroot().xpath(
+        vhd_value = json.loads(etree.parse(StringIO(data)).getroot().xpath(
             "./Object/Property[@Name='StorageProfileText']/text()")[0])
-        vhd_disk_uri = status_value['OSDisk']['VirtualHardDisk']['Uri']
-        self.logger.info("Returned Disk Endpoint was {}".format(vhd_disk_uri))
-        return os.path.split(urlparse.urlparse(vhd_disk_uri).path)[1]
+        vhd_endpoint = vhd_value['OSDisk']['VirtualHardDisk']['Uri']
+        self.logger.info("Returned Disk Endpoint was {}".format(vhd_endpoint))
+        return vhd_endpoint
 
     def get_network_interface(self, vm_name, resource_group=None):
         self.logger.info("Attempting to Retrieve Azure VM Network Interface {}".format(vm_name))
