@@ -8,7 +8,7 @@ from apiclient.discovery import build
 from apiclient.http import MediaFileUpload
 from apiclient import errors
 from base import MgmtSystemAPIBase, VMInfo
-from exceptions import VMInstanceNotFound, ImageNotFoundError
+from exceptions import VMInstanceNotFound, ImageNotFoundError, ActionNotSupported
 from json import dumps as json_dumps
 from oauth2client.service_account import ServiceAccountCredentials
 from wait_for import wait_for
@@ -31,7 +31,8 @@ DEFAULT_MIMETYPE = 'application/octet-stream'
 IMAGE_PROJECTS = ['centos-cloud', 'debian-cloud', 'rhel-cloud', 'suse-cloud', 'ubuntu-os-cloud',
                 'windows-cloud', 'opensuse-cloud', 'coreos-cloud', 'google-containers']
 
-class GoogleCloudSystem (MgmtSystemAPIBase):
+
+class GoogleCloudSystem(MgmtSystemAPIBase):
     """
     Client to Google Cloud Platform API
 
@@ -276,20 +277,17 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
         self.logger.info(json_dumps(response, indent=2))
         return (True, blob_name)
 
-    def create_vm(self, instance_name, source_disk_image=None, machine_type=None,
-            startup_script_data=None, timeout=180):
-        if self.does_vm_exist(instance_name):
-            self.logger.info("The {} instance is already exists, skipping".format(instance_name))
-            return True
+    def deploy_template(self, template, **kwargs):
 
+        template_link = self.get_image_by_name(template)['selfLink']
+
+        instance_name = kwargs['vm_name']
         self.logger.info("Creating {} instance".format(instance_name))
 
-        if not source_disk_image:
-            source_disk_image = "projects/debian-cloud/global/images/debian-7-wheezy-v20150320"
-
-        machine_type = machine_type or ("zones/{}/machineTypes/n1-standard-1".format(self._zone))
-
-        script = startup_script_data or "#!/bin/bash"
+        machine_type = kwargs.get('machine_type',
+            "zones/{}/machineTypes/n1-standard-1".format(self._zone))
+        script = kwargs.get('startup_script_data', "#!/bin/bash")
+        timeout = kwargs.get('timeout', 180)
 
         config = {
             'name': instance_name,
@@ -301,7 +299,7 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
                     'boot': True,
                     'autoDelete': True,
                     'initializeParams': {
-                        'sourceImage': source_disk_image,
+                        'sourceImage': template_link,
                     }
                 }
             ],
@@ -350,6 +348,9 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
             num_sec=timeout, message=" Create {}".format(instance_name))
         return True
 
+    def create_vm(self):
+        raise NotImplementedError('create_vm not implemented.')
+
     def delete_vm(self, instance_name, timeout=180):
         if not self.does_vm_exist(instance_name):
             self.logger.info("The {} instance is not exists, skipping".format(instance_name))
@@ -380,7 +381,7 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
         operation = self._instances.stop(
             project=self._project, zone=self._zone, instance=instance_name).execute()
         wait_for(lambda: self._nested_operation_wait(operation['name']),
-            message="Stop {}".format(instance_name))
+            message="Stop {}".format(instance_name), timeout=360)
         return True
 
     def start_vm(self, instance_name):
@@ -403,10 +404,13 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
 
     # Get external IP (ephemeral)
     def current_ip_address(self, vm_name):
-        return self.vm_status(vm_name)['natIP']
-
-    def deploy_template(self, template, *args, **kwargs):
-        raise NotImplementedError('deploy_template not implemented.')
+        zones = self._compute.zones().list(project=self._project).execute()
+        for zone in zones.get('items', []):
+            zone_name = zone.get('name', None)
+            for vm in self._get_zone_instances(zone_name).get('items', []):
+                if vm['name'] == vm_name:
+                    access_configs = vm.get('networkInterfaces')[0].get('accessConfigs')[0]
+                    return access_configs.get('natIP')
 
     def disconnect(self):
         """Disconnect from the GCE
@@ -435,7 +439,7 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
         return self.vm_status(vm_name) in self.states['stopped']
 
     def is_vm_suspended(self, vm_name):
-        raise NotImplementedError('is_vm_suspended not implemented.')
+        raise ActionNotSupported('vm_suspend not supported.')
 
     # These methods indicate if the vm is in the process of stopping or starting
     def is_vm_stopping(self, vm_name):
@@ -454,7 +458,7 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
         raise NotImplementedError('remove_host_from_cluster not implemented.')
 
     def suspend_vm(self, vm_name):
-        raise NotImplementedError('suspend_vm not implemented.')
+        raise ActionNotSupported('vm_suspend not supported.')
 
     def vm_status(self, vm_name):
         if self.does_vm_exist(vm_name):
@@ -470,7 +474,7 @@ class GoogleCloudSystem (MgmtSystemAPIBase):
         wait_for(self.is_vm_stopped, [vm_name], num_sec=num_sec)
 
     def wait_vm_suspended(self, vm_name, num_sec):
-        raise NotImplementedError('wait_vm_suspended not implemented.')
+        raise ActionNotSupported('vm_suspend not supported.')
 
     def all_vms(self):
         result = []
