@@ -74,6 +74,11 @@ class RHEVMSystem(MgmtSystemAPIBase):
         username: The username to connect with.
         password: The password to connect with.
 
+    Keywords:
+        port: (Optional) Port where RHEVM API listens.
+        api_endpoint: (Optional) If you need to fine-tune and pass an exact endpoint in form of a
+            full URL, use this keyword. the ``port`` keyword is then not used.
+
     Returns: A :py:class:`RHEVMSystem` object.
     """
 
@@ -90,10 +95,13 @@ class RHEVMSystem(MgmtSystemAPIBase):
     def __init__(self, hostname, username, password, **kwargs):
         # generate URL from hostname
         super(RHEVMSystem, self).__init__(kwargs)
-        if 'port' in kwargs:
-            url = 'https://%s:%s/api' % (hostname, kwargs['port'])
+        url_component = 'ovirt-engine/api' if float(kwargs['version']) >= 4.0 else 'api'
+        if 'api_endpoint' in kwargs:
+            url = kwargs['api_endpoint']
+        elif 'port' in kwargs:
+            url = 'https://{}:{}/{}'.format(hostname, kwargs['port'], url_component)
         else:
-            url = 'https://%s/api' % hostname
+            url = 'https://{}/{}'.format(hostname, url_component)
 
         self._api = None
         self._api_kwargs = {
@@ -296,6 +304,9 @@ class RHEVMSystem(MgmtSystemAPIBase):
             )
         return result
 
+    def get_vm_guid(self, vm_name):
+        return self._get_vm(vm_name).get_id()
+
     def list_host(self, **kwargs):
         host_list = self.api.hosts.list(**kwargs)
         return [host.name for host in host_list]
