@@ -11,7 +11,7 @@ except AttributeError:
     pass
 
 import re
-from datetime import datetime
+from datetime import datetime, timedelta
 from distutils.version import LooseVersion
 from functools import partial
 import operator
@@ -295,6 +295,13 @@ class VMWareSystem(MgmtSystemAPIBase):
             )
         return result
 
+    def get_vm_guid(self, vm_name):
+        vm = self._get_vm(vm_name)
+        try:
+            return str(vm.summary.config.uuid)
+        except AttributeError:
+            return None
+
     def get_vm_name_from_ip(self, ip):
         """ Gets the name of a vm from its IP.
 
@@ -387,6 +394,10 @@ class VMWareSystem(MgmtSystemAPIBase):
     def list_host(self):
         return [str(h.name) for h in mobs.HostSystem.all(self.api)]
 
+    def list_host_datastore_url(self, host_name):
+        host = mobs.HostSystem.get(self.api, name=host_name)
+        return [str(d.summary.url) for d in host.datastore]
+
     def list_datastore(self):
         return [str(h.name) for h in mobs.Datastore.all(self.api) if h.host]
 
@@ -409,9 +420,11 @@ class VMWareSystem(MgmtSystemAPIBase):
         return self._get_vm(vm_name, force=True).runtime.powerState
 
     def vm_creation_time(self, vm_name):
-        # psphere turns date strings in datetime for us
         vm = self._get_vm(vm_name)
-        return vm.runtime.bootTime
+        vcenter_time_now = self.api.si.CurrentTime()
+        vm_uptime = vm.summary.quickStats.uptimeSeconds
+        vm_delta = timedelta(seconds=int(vm_uptime))
+        return vcenter_time_now - vm_delta
 
     def get_vm_host_name(self, vm_name):
         vm = self._get_vm(vm_name)
@@ -497,7 +510,7 @@ class VMWareSystem(MgmtSystemAPIBase):
 
     def clone_vm(self, source, destination, resourcepool=None, datastore=None, power_on=True,
                  sparse=False, template=False, provision_timeout=1800, progress_callback=None,
-                 allowed_datastores=None, cpu=None, ram=None):
+                 allowed_datastores=None, cpu=None, ram=None, **kwargs):
         try:
             if mobs.VirtualMachine.get(self.api, name=destination).name == destination:
                 raise Exception("VM already present!")
